/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.service;

import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.User;

/**
 * Class for manipulating with user entity. Analogous to DAO pattern.
 *  EJB3 Bean
 *
 * @author Andrey Markhel
 */

@Name("userAction")
@Stateless
@AutoCreate
public class UserAction implements IUserAction {

	@In(value="entityManager")
	EntityManager em;
	
	@In private User user;

	/**
     * Login user. If succes return logged user, otherwise return null
     * @param username - username
     * @param password - password
     * @return user if success
     */
	public User login(String username, String password) {
		return (User)em.createNamedQuery(Constants.USER_LOGIN_QUERY)
		.setParameter(Constants.USERNAME_PARAMETER, username)
		.setParameter(Constants.PASSWORD_PARAMETER, password)
		.getSingleResult();
	}

	/**
     * Persist user entity to database
     * @param user - user to register
     */
	public void register(User user) {
		em.persist(user);
		em.flush();
	}
	
	/**
     * Synchronize state of user entity with database
     * @return user if success
     */
	public User updateUser() {
		em.flush();
		return user;
	}
	
	/**
     * Check if user with specified login already exist
     * @return is user with specified login already exist
     */
	public boolean isUserExist(String login) {
		return em.createNamedQuery(Constants.USER_EXIST_QUERY)
		.setParameter(Constants.LOGIN_PARAMETER, login)
		.getResultList().size() != 0;
	}
	
	/**
     * Used to retrieve all shared albums, belongs to specified user.
     * @param user
     * @return list of shared albums of specified user
     */
	@SuppressWarnings("unchecked")
	public List<Album> getSharedAlbums(User user){
		return (List<Album>)em.createNamedQuery(Constants.USER_SHARED_ALBUMS_QUERY).setParameter(Constants.USER_PARAMETER, user).getResultList();
	}
	
	/**
     * Used to retrieve all shared images, belongs to specified user.
     * @param user
     * @return list of shared images of specified user
     */
	@SuppressWarnings("unchecked")
	public List<Image> getSharedImages(User user){
		return (List<Image>)em.createNamedQuery(Constants.USER_SHARED_IMAGES_QUERY).setParameter(Constants.USER_PARAMETER, user).getResultList();
	}
	
	/**
     * Used to retrieve count of all albums, belongs to specified user.
     * @return count of all albums of current user
     */
	public Long countAlbums(){
		return (Long)em.createNamedQuery(Constants.USER_COUNT_ALBUMS_QUERY).setParameter(Constants.USER_PARAMETER, user).getSingleResult();
	}
	
	/**
     * Used to retrieve count of shared albums, belongs to specified user.
     * @param user
     * @return count of shared albums of specified user
     */
	public Long countSharedAlbums(User user){
		return (Long)em.createNamedQuery(Constants.USER_COUNT_SHARED_ALBUMS_QUERY).setParameter(Constants.USER_PARAMETER, user).getSingleResult();
	}
	
	/**
     * Used to retrieve count of all images, belongs to specified user.
     * @return count of all images of current user
     */
	public Long countImages(){
		return (Long)em.createNamedQuery(Constants.USER_COUNT_IMAGES_QUERY).setParameter(Constants.USER_PARAMETER, user).getSingleResult();
	}
	
	/**
     * Used to retrieve count shared images, belongs to specified user.
     * @param user
     * @return count of shared images of specified user
     */
	public Long countSharedImages(User user){
		return (Long)em.createNamedQuery(Constants.USER_COUNT_SHARED_IMAGES_QUERY).setParameter(Constants.USER_PARAMETER, user).getSingleResult();
	}
}